@@ -4,7 +4,6 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 07:01:42 1999
-# Version       : $Id: BinariesCheck.py 1865 2011-05-15 09:05:04Z scop $
 # Purpose       : check binary files in a binary rpm package.
 #############################################################################
 
@@ -23,6 +22,12 @@ DEFAULT_SYSTEM_LIB_PATHS = (
     '/lib', '/usr/lib', '/usr/X11R6/lib',
     '/lib64', '/usr/lib64', '/usr/X11R6/lib64')
 
+def create_regexp_call(call):
+    if type(call) == type([]):
+        call = '(?:' + '|'.join(call) + ')'
+    r = "\s+FUNC\s+.*?\s+(%s(?:@GLIBC\S+)?)(?:\s|$)" % call
+    return re.compile(r)
+
 class BinaryInfo:
 
     needed_regex = re.compile('\s+\(NEEDED\).*\[(\S+)\]')
@@ -35,8 +40,15 @@ class BinaryInfo:
     stack_exec_regex = re.compile('^..E$')
     undef_regex = re.compile('^undefined symbol:\s+(\S+)')
     unused_regex = re.compile('^\s+(\S+)')
-    exit_call_regex = re.compile('\s+FUNC\s+.*?\s+(_?exit(?:@\S+)?)(?:\s|$)')
-    fork_call_regex = re.compile('\s+FUNC\s+.*?\s+(fork(?:@\S+)?)(?:\s|$)')
+    exit_call_regex = create_regexp_call('_?exit')
+    fork_call_regex = create_regexp_call('fork')
+    # regexp for setgid setegid setresgid set(?:res|e)?gid
+    setgid_call_regex = create_regexp_call(['setresgid','setegid','setgid'])
+    setuid_call_regex = create_regexp_call(['setresuid','seteuid','setuid'])
+    setgroups_call_regex = create_regexp_call(['initgroups','setgroups'])
+    chroot_call_regex = create_regexp_call('chroot')
+    chdir_call_regex  = create_regexp_call('chdir')
+    mktemp_call_regex = create_regexp_call('mktemp')
 
     def __init__(self, pkg, path, file, is_ar, is_shlib):
         self.readelf_error = False
@@ -53,6 +65,14 @@ class BinaryInfo:
         fork_called = False
         self.tail = ''
 
+        self.setgid = False
+        self.setuid = False
+        self.setgroups = False
+        self.chroot = False
+        self.chdir = False
+        self.chroot_near_chdir = False
+        self.mktemp = False
+
         is_debug = path.endswith('.debug')
 
         cmd = ['env', 'LC_ALL=C', 'readelf', '-W', '-S', '-l', '-d', '-s']
@@ -61,6 +81,24 @@ class BinaryInfo:
         if not res[0]:
             for l in res[1].splitlines():
 
+                if BinaryInfo.mktemp_call_regex.search(l):
+                    self.mktemp = True
+
+                if BinaryInfo.setgid_call_regex.search(l):
+                    self.setgid = True
+
+                if BinaryInfo.setuid_call_regex.search(l):
+                    self.setuid = True
+
+                if BinaryInfo.setgroups_call_regex.search(l):
+                    self.setgroups = True
+
+                if BinaryInfo.chdir_call_regex.search(l):
+                    self.chdir = True
+
+                if BinaryInfo.chroot_call_regex.search(l):
+                    self.chroot = True
+
                 r = BinaryInfo.needed_regex.search(l)
                 if r:
                     self.needed.append(r.group(1))
@@ -112,6 +150,23 @@ class BinaryInfo:
             if fork_called:
                 self.exit_calls = []
 
+            # check if chroot is near chdir ( since otherwise, chroot is called without chdir )
+            if self.chroot and self.chdir:
+                # FIXME this check is too slow, because forking for objdump is quite slow
+                # according to a quick test and that's quite visible on a server like postfix
+                res = Pkg.getstatusoutput(('env', 'LC_ALL=C', 'objdump', '-d', path))
+                if not res[0]:
+                    call = []
+                    # we want that :
+                    # 401eb8:   e8 c3 f0 ff ff          callq  400f80 <free@plt>
+                    for l in res[1].splitlines():
+                        if l.find('callq ') >= 0:
+                            call.append(l.rpartition(' ')[2])
+                    for index,c in enumerate(call):
+                        if c.find('chroot@plt') >= 0:
+                            for i in call[index-2:index+2]:
+                                if i.find('chdir@plt'):
+                                    self.chroot_near_chdir = True
         else:
             self.readelf_error = True
             printWarning(pkg, 'binaryinfo-readelf-failed',
@@ -171,14 +226,14 @@ class BinaryInfo:
 path_regex = re.compile('(.*/)([^/]+)')
 numeric_dir_regex = re.compile('/usr(?:/share)/man/man./(.*)\.[0-9](?:\.gz|\.bz2)')
 versioned_dir_regex = re.compile('[^.][0-9]')
-libc_regex = re.compile('libc\.')
 ldso_soname_regex = re.compile('^ld(-linux(-(ia|x86_)64))?\.so')
 so_regex = re.compile('/lib(64)?/[^/]+\.so(\.[0-9]+)*$')
 validso_regex = re.compile('(\.so\.\d+(\.\d+)*|\d\.so)$')
 sparc_regex = re.compile('SPARC32PLUS|SPARC V9|UltraSPARC')
 system_lib_paths = Config.getOption('SystemLibPaths', DEFAULT_SYSTEM_LIB_PATHS)
 pie_exec_re = Config.getOption('PieExecutables')
-if pie_exec_re: pie_exec_re = re.compile(pie_exec_re)
+if pie_exec_re:
+    pie_exec_re = re.compile(pie_exec_re)
 usr_lib_regex = re.compile('^/usr/lib(64)?/')
 bin_regex = re.compile('^(/usr(/X11R6)?)?/s?bin/')
 soversion_regex = re.compile('.*?([0-9][.0-9]*)\\.so|.*\\.so\\.([0-9][.0-9]*).*')
@@ -231,7 +286,8 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
             is_elf = 'ELF' in pkgfile.magic
             is_ar = 'current ar archive' in pkgfile.magic
             is_ocaml_native = 'Objective caml native' in pkgfile.magic
-            is_binary = is_elf or is_ar or is_ocaml_native
+            is_lua_bytecode = 'Lua bytecode' in pkgfile.magic
+            is_binary = is_elf or is_ar or is_ocaml_native or is_lua_bytecode
 
             if not is_binary:
                 if reference_regex.search(fname):
@@ -269,7 +325,7 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
             if pkg.arch == 'sparc' and sparc_regex.search(pkgfile.magic):
                 printError(pkg, 'non-sparc32-binary', fname)
 
-            if is_ocaml_native or fname.endswith('.o') or \
+            if is_ocaml_native or is_lua_bytecode or fname.endswith('.o') or \
                     fname.endswith('.static'):
                 continue
 
@@ -375,14 +431,14 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
 
             else:
                 # linked against libc ?
-                if not libc_regex.search(fname) and \
+                if "libc." not in fname and \
                         (not bin_info.soname or \
-                             (not libc_regex.search(bin_info.soname) and \
+                             ("libc." not in bin_info.soname and \
                               not ldso_soname_regex.search(bin_info.soname))):
 
                     found_libc = False
                     for lib in bin_info.needed:
-                        if libc_regex.search(lib):
+                        if "libc." in lib:
                             found_libc = True
                             break
 
@@ -402,6 +458,16 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
                 pkg.arch in ("athlon", "x86_64")):
                 printError(pkg, 'missing-PT_GNU_STACK-section', fname)
 
+            if bin_info.setgid and bin_info.setuid and not bin_info.setgroups:
+                printError(pkg, 'missing-call-to-setgroups', fname)
+
+            if bin_info.chroot:
+                if not bin_info.chdir or not bin_info.chroot_near_chdir:
+                    printError(pkg, 'missing-call-to-chdir-with-chroot', fname)
+
+            if bin_info.mktemp:
+                printError(pkg, 'call-to-mktemp', fname)
+
         if has_lib:
             for f in exec_files:
                 printError(pkg, 'executable-in-library-package', f)
@@ -557,6 +623,31 @@ http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=256900#49''',
 'non-position-independent-executable',
 '''This executable must be position independent.  Check that it is built with
 -fPIE/-fpie in compiler flags and -pie in linker flags.''',
+
+'missing-call-to-setgroups-before-setuid',
+'''This executable is calling setuid and setgid without setgroups or initgroups.
+There is a high probability this mean it didn't relinquish all groups, and this
+would be a potential security issue to be fixed. Seek POS36-C on the web for
+details about the problem.''',
+
+'missing-call-to-chdir-with-chroot',
+'''This executable appears to call chroot without using chdir to change the
+current directory. This is likely an error and permits an attacker to break out
+of the chroot by using fchdir. While that's not always a security issue, this
+has to be checked.''',
+
+'call-to-mktemp',
+'''This executable calls mktemp. As advised by the manpage (mktemp(3)), this
+function should be avoided. Some implementations are deeply insecure, and there
+is a race condition between the time of check and time of use (TOCTOU).
+See http://capec.mitre.org/data/definitions/29.html for details, and contact
+upstream to have this issue fixed.''',
+
+'unstripped-binary-or-object',
+'''This executable should be stripped from debugging symbols, in order to take
+less space and be loaded faster. This is usually done automatically at buildtime
+by rpm. Check the build logs and the permission on the file (some
+implementations only strip if the permission is 0755).'''
 )
 
 # BinariesCheck.py ends here
