@@ -4,7 +4,6 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Mon Oct  4 19:32:49 1999
-# Version       : $Id: FilesCheck.py 1863 2011-05-13 17:10:53Z scop $
 # Purpose       : test various aspects on files: locations, owner, groups,
 #                 permission, setuid, setgid...
 #############################################################################
@@ -47,6 +46,7 @@ STANDARD_DIRS = (
     '/opt',
     '/proc',
     '/root',
+    '/run',
     '/sbin',
     '/selinux',
     '/srv',
@@ -169,7 +169,7 @@ DEFAULT_STANDARD_GROUPS = ('root', 'bin', 'daemon', 'adm', 'lp', 'sync',
                            'shutdown', 'halt', 'mail', 'news', 'uucp',
                            'man', 'nobody',)
 
-tmp_regex = re.compile('^/tmp/|^(/var|/usr)/tmp/')
+tmp_regex = re.compile('^(/var|/usr)?/tmp/')
 sub_bin_regex = re.compile('^(/usr)?/s?bin/\S+/')
 backup_regex = re.compile('(~|\#[^/]+\#|\.orig|\.rej)$')
 compr_regex = re.compile('\.(gz|z|Z|zip|bz2|lzma|xz)$')
@@ -220,6 +220,8 @@ skipdocs_regex = re.compile(Config.getOption('SkipDocsRegexp', '\.(?:rtf|x?html?
 meta_package_regex = re.compile(Config.getOption('MetaPackageRegexp', '^(bundle|task)-'))
 filesys_packages = ['filesystem'] # TODO: make configurable?
 quotes_regex = re.compile('[\'"]+')
+start_certificate_regex = re.compile('^-----BEGIN CERTIFICATE-----$')
+start_private_key_regexp = re.compile('^----BEGIN PRIVATE KEY-----$')
 
 for idx in range(0, len(dangling_exceptions)):
     dangling_exceptions[idx][0] = re.compile(dangling_exceptions[idx][0])
@@ -233,7 +235,7 @@ standard_users = Config.getOption('StandardUsers', DEFAULT_STANDARD_USERS)
 non_readable_regexs = (re.compile('^/var/log/'),
                        re.compile('^/etc/(g?shadow-?|securetty)$'))
 
-man_base_regex = re.compile(r'^/usr(?:/share)?/man/man[^/]+/(.+)\.[1-9n]')
+man_base_regex = re.compile(r'^/usr(?:/share)?/man(?:/overrides)?/man[^/]+/(.+)\.[1-9n]')
 man_warn_regex = re.compile(r'^([^:]+:)\d+:\s*')
 man_nowarn_regex = re.compile(
     # From Lintian: ignore common undefined macros from pod2man << Perl 5.10
@@ -247,6 +249,8 @@ man_warn_category = Config.getOption('ManWarningCategory', 'mac')
 fsf_license_regex = re.compile('(GNU((\s+(Library|Lesser|Affero))?(\s+General)?\s+Public|\s+Free\s+Documentation)\s+Licen[cs]e|(GP|FD)L)', re.IGNORECASE)
 fsf_wrong_address_regex = re.compile('(675\s+Mass\s+Ave|59\s+Temple\s+Place|Franklin\s+Steet|02139|02111-1307)', re.IGNORECASE)
 
+scalable_icon_regex = re.compile(r'^/usr(?:/local)?/share/icons/.*/scalable/')
+
 # loosely inspired from Python Cookbook
 # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/173220
 text_characters = "".join(map(chr, range(32, 127)) + list("\n\r\t\b"))
@@ -261,7 +265,7 @@ def peek(filename, pkg, length=1024):
         fobj = open(filename, 'rb')
         chunk = fobj.read(length)
         fobj.close()
-    except Exception, e: # eg. https://bugzilla.redhat.com/209876
+    except IOError, e: # eg. https://bugzilla.redhat.com/209876
         printWarning(pkg, 'read-error', e)
         if fobj:
             fobj.close()
@@ -273,8 +277,13 @@ def peek(filename, pkg, length=1024):
     if not chunk:  # Empty files are considered text
         return (chunk, True)
 
+    fl = filename.lower()
+
     # PDF's are binary but often detected as text by the algorithm below
-    if filename.lower().endswith('.pdf') and chunk.startswith('%PDF-'):
+    if fl.endswith('.pdf') and chunk.startswith('%PDF-'):
+        return (chunk, False)
+    # Ditto RDoc RI files
+    if fl.endswith('.ri') and '/ri/' in fl:
         return (chunk, False)
 
     # Get the non-text characters (maps a character to itself then
@@ -285,8 +294,9 @@ def peek(filename, pkg, length=1024):
     istext = float(len(t))/len(chunk) <= 0.30
     return (chunk, istext)
 
-# See Python/import.c (in the trunk and py3k branches) for a full list of
-# the values here.
+# See Python sources for a full list of the values here.
+# http://hg.python.org/cpython/file/tip/Lib/importlib/_bootstrap.py
+# http://hg.python.org/cpython/file/2.7/Python/import.c
 _python_magic_values = {
     '2.2': 60717,
     '2.3': 62011,
@@ -297,6 +307,8 @@ _python_magic_values = {
     '3.0': 3130,
     '3.1': 3150,
     '3.2': 3180,
+    '3.3': 3230,
+    '3.4': 3260,
     }
 
 def get_expected_pyc_magic(path):
@@ -433,16 +445,13 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     is_kernel_package:
                 printError(pkg, "kernel-modules-not-in-kernel-packages", f)
 
+            for i in ['mnt','opt','usr-local','var-local','home']:
+                if f.startswith('/%s/' % i.replace('-','/')):
+                    printError(pkg, 'dir-or-file-in-%s' % i, f)
+
             if tmp_regex.search(f):
                 printError(pkg, 'dir-or-file-in-tmp', f)
-            elif f.startswith('/mnt/'):
-                printError(pkg, 'dir-or-file-in-mnt', f)
-            elif f.startswith('/opt/'):
-                printError(pkg, 'dir-or-file-in-opt', f)
-            elif f.startswith('/usr/local/'):
-                printError(pkg, 'dir-or-file-in-usr-local', f)
-            elif f.startswith('/var/local/'):
-                printError(pkg, 'dir-or-file-in-var-local', f)
+
             elif f.startswith('/var/run/'):
                 if f not in ghost_files:
                     printWarning(pkg, 'non-ghost-in-var-run', f)
@@ -451,8 +460,6 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     printWarning(pkg, 'non-ghost-in-var-lock', f)
             elif sub_bin_regex.search(f):
                 printError(pkg, 'subdir-in-bin', f)
-            elif f.startswith('/home/'):
-                printError(pkg, 'dir-or-file-in-home', f)
             elif '/site_perl/' in f:
                 printWarning(pkg, 'siteperl-in-perl-module', f)
 
@@ -512,15 +519,14 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                    oct(perm))
 
                 # Prefetch scriptlets, strip quotes from them (#169)
-                postin = pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
+                postin = pkg[rpm.RPMTAG_POSTIN] or \
+                    pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
                 if postin:
                     postin = quotes_regex.sub('', postin)
-                postun = pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
+                postun = pkg[rpm.RPMTAG_POSTUN] or \
+                    pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
                 if postun:
                     postun = quotes_regex.sub('', postun)
-                preun = pkg[rpm.RPMTAG_PREUN] or pkg[rpm.RPMTAG_PREUNPROG]
-                if preun:
-                    preun = quotes_regex.sub('', preun)
 
                 if not devel_pkg:
                     if lib_path_regex.search(f):
@@ -598,6 +604,8 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     elif not install_info_regex.search(postin):
                         printError(pkg, 'postin-without-install-info', f)
 
+                    preun = pkg[rpm.RPMTAG_PREUN] or \
+                        pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
                     if not postun and not preun:
                         printError(pkg,
                                    'info-files-without-install-info-postun', f)
@@ -748,7 +756,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                 res = man_base_regex.search(f)
                 if res:
                     man_basenames.add(res.group(1))
-                    if use_utf8:
+                    if use_utf8 and chunk:
                         # TODO: better shell escaping or seq based invocation
                         cmd = commands.getstatusoutput(
                             'env LC_ALL=C %s "%s" | gtbl | '
@@ -762,6 +770,17 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                             printWarning(pkg, "manual-page-warning", f,
                                          line[res.end(1):])
 
+                if f.endswith(".svgz") and f[0:-1] not in files \
+                        and scalable_icon_regex.search(f):
+                    printWarning(pkg, "gzipped-svg-icon", f)
+
+                if f.endswith('.pem'):
+                    if f not in ghost_files:
+                        if pkg.grep(start_certificate_regex, f):
+                            printWarning(pkg, 'pem-certificate', f)
+                        if pkg.grep(start_private_key_regexp, f):
+                            printError(pkg, 'pem-private-key', f)
+
                 # text file checks
                 if istext:
                     # ignore perl module shebang -- TODO: disputed...
@@ -807,7 +826,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                             fsf_wrong_address_regex.search(chunk):
                         printError(pkg, 'incorrect-fsf-address', f)
 
-                elif is_doc and compr_regex.search(f):
+                elif is_doc and chunk and compr_regex.search(f):
                     ff = compr_regex.sub('', f)
                     if not skipdocs_regex.search(ff):
                         # compressed docs, eg. info and man files etc
@@ -1328,6 +1347,23 @@ as intended.''',
 '''The Free Software Foundation address in this file seems to be outdated or
 misspelled.  Ask upstream to update the address, or if this is a license file,
 possibly the entire file with a new copy available from the FSF.''',
+
+'gzipped-svg-icon',
+'''Not all desktop environments that support SVG icons support them gzipped
+(.svgz).  Install the icon as plain uncompressed SVG.''',
+
+'pem-certificate',
+'''Shipping a PEM certificate is likely wrong. If used for the default
+configuration, this is insecure ( since the certificate is public ). If this
+is used for validation, ie a CA certificate store, then this must be kept up
+to date due to CA compromise. The only valid reason is for testing purpose,
+so ignore this warning if this is the case.''',
+
+'pem-private-key',
+'''Private key in a .pem file should not be shipped in a rpm, unless
+this is for testing purpose ( ie, run by the test suite ). Shipping it
+as part of the example documentation mean that someone will sooner or later
+use it and setup a insecure configuration.'''
 )
 
 # FilesCheck.py ends here
