@@ -4,7 +4,6 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 07:18:06 1999
-# Version       : $Id: Pkg.py 1848 2011-04-06 20:22:19Z scop $
 # Purpose       : provide an API to handle a rpm package either by accessing
 #                 the rpm file or by accessing the files contained inside.
 #############################################################################
@@ -12,7 +11,6 @@
 import commands
 import os
 import re
-import stat
 import subprocess
 import sys
 import tempfile
@@ -88,7 +86,8 @@ def getstatusoutput(cmd, stdoutonly = False):
     proc.stdin.close()
     text = proc.stdout.read()
     sts = proc.wait()
-    if sts is None: sts = 0
+    if sts is None:
+        sts = 0
     if text.endswith('\n'):
         text = text[:-1]
     return sts, text
@@ -478,8 +477,7 @@ class Pkg:
 
     # extract rpm contents
     def _extract(self):
-        s = os.stat(self.dirname)
-        if not stat.S_ISDIR(s[stat.ST_MODE]):
+        if not os.path.isdir(self.dirname):
             warn('Unable to access dir %s' % self.dirname)
             return None
         else:
@@ -748,6 +746,18 @@ class Pkg:
                              rpm.RPMTAG_OBSOLETEFLAGS,
                              rpm.RPMTAG_OBSOLETEVERSION)
 
+    def scriptprog(self, which):
+        """Get the specified script interpreter as a string.
+           Depending on rpm-python version, the string may or may not include
+           interpreter arguments, if any."""
+        prog = self[which]
+        if prog is None:
+            prog = ""
+        elif not isinstance(prog, basestring):
+            # http://rpm.org/ticket/847#comment:2
+            prog = " ".join(prog)
+        return prog
+
 def getInstalledPkgs(name):
     """Get list of installed package objects by name."""
 
