@@ -4,7 +4,6 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Thu Oct  7 17:06:14 1999
-# Version       : $Id: SpecCheck.py 1867 2011-05-31 20:37:45Z scop $
 # Purpose       : check the spec file of a source rpm.
 #############################################################################
 
@@ -21,7 +20,7 @@ from TagsCheck import VALID_GROUPS
 import AbstractCheck
 import Config
 import Pkg
-
+import Common
 
 # Don't check for hardcoded library paths in biarch packages
 DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
@@ -30,16 +29,28 @@ DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
 # their noarch files in /usr/lib/<package>/*, or packages that can't
 # be installed on biarch systems
 DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS = '/lib/(modules|cpp|perl5|rpm|hotplug|firmware)($|[\s/,])'
-patch_regex = re.compile("^Patch(\d*)\s*:\s*(\S+)", re.IGNORECASE)
+
+def re_tag_compile(tag):
+    if type(tag) == type([]):
+        tag = '(?:' + '|'.join(tag) + ')'
+    r = "^%s\s*:\s*(\S+)\s*$" % tag
+    return re.compile(r, re.IGNORECASE)
+
+
+
+patch_regex = re_tag_compile('Patch(\d*)')
 applied_patch_regex = re.compile("^%patch(\d*)")
 applied_patch_p_regex = re.compile("\s-P\s+(\d+)\\b")
 applied_patch_pipe_regex = re.compile(r'\s%\{PATCH(\d+)\}\s*\|\s*(%\{?__)?patch\b')
 source_dir_regex = re.compile("^[^#]*(\$RPM_SOURCE_DIR|%{?_sourcedir}?)")
-obsolete_tags_regex = re.compile("^(Copyright|Serial)\s*:\s*(\S+)")
-buildroot_regex = re.compile('^BuildRoot\s*:\s*(\S+)', re.IGNORECASE)
-prefix_regex = re.compile('^Prefix\s*:\s*(\S+)', re.IGNORECASE)
-packager_regex = re.compile('^Packager\s*:\s*(\S+)', re.IGNORECASE)
-noarch_regex = re.compile('^BuildArch(?:itectures)?\s*:\s*\\bnoarch\\b', re.IGNORECASE)
+obsolete_tags_regex = re_tag_compile(['Serial','Copyright'])
+buildroot_regex = re_tag_compile('BuildRoot')
+prefix_regex = re_tag_compile('Prefix')
+packager_regex = re_tag_compile('Packager')
+buildarch_regex = re_tag_compile(['BuildArch','BuildArchitectures'])
+buildprereq_regex = re_tag_compile('BuildPreReq')
+prereq_regex = re_tag_compile('PreReq(\(.*\))')
+
 make_check_regex = re.compile('(^|\s|%{?__)make}?\s+(check|test)')
 rm_regex = re.compile('(^|\s)((.*/)?rm|%{?__rm}?) ')
 rpm_buildroot_regex = re.compile('^[^#]*(?:(\\\*)\${?RPM_BUILD_ROOT}?|(%+){?buildroot}?)')
@@ -50,8 +61,6 @@ if_regex = re.compile('^\s*%if\s')
 endif_regex = re.compile('^\s*%endif\\b')
 biarch_package_regex = re.compile(DEFAULT_BIARCH_PACKAGES)
 hardcoded_lib_path_exceptions_regex = re.compile(Config.getOption('HardcodedLibPathExceptions', DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS))
-prereq_regex = re.compile('^PreReq(\(.*\))?:\s*(.+?)\s*$', re.IGNORECASE)
-buildprereq_regex = re.compile('^BuildPreReq:\s*(.+?)\s*$', re.IGNORECASE)
 use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
 libdir_regex = re.compile('%{?_lib(?:dir)?\}?\\b')
 comment_or_empty_regex = re.compile('^\s*(#|$)')
@@ -59,10 +68,8 @@ defattr_regex = re.compile('^\s*%defattr\\b')
 attr_regex = re.compile('^\s*%attr\\b')
 section_regexs = dict(
     ([x, re.compile('^%' + x + '(?:\s|$)')]
-     for x in ('build', 'changelog', 'check', 'clean', 'description', 'files',
-               'install', 'package', 'prep', 'pre', 'post', 'preun', 'postun',
-               'trigger', 'triggerin', 'triggerun', 'triggerprein',
-               'triggerpostun', 'pretrans', 'posttrans')))
+     for x in ['build', 'changelog', 'check', 'clean', 'description', 'files',
+               'install', 'package', 'prep'] + Common.RPM_SCRIPTLETS))
 deprecated_grep_regex = re.compile(r'\b[ef]grep\b')
 
 # Only check for /lib, /usr/lib, /usr/X11R6/lib
@@ -74,8 +81,10 @@ hardcoded_library_path_regex = re.compile('^[^#]*((^|\s+|\.\./\.\.|\${?RPM_BUILD
 # https://bugzilla.redhat.com/118780 and bugs linked to that one.
 scriptlet_requires_regex = re.compile('^(PreReq|Requires)\([^\)]*,', re.IGNORECASE)
 
-depscript_override_regex = re.compile('(^|\s)%(define|global)\s+__find_(requires|provides)\s')
-depgen_disable_regex = re.compile('(^|\s)%(define|global)\s+_use_internal_dependency_generator\s+0')
+DEFINE_RE='(^|\s)%(define|global)\s+'
+depscript_override_regex = re.compile(DEFINE_RE + '__find_(requires|provides)\s')
+depgen_disable_regex = re.compile(DEFINE_RE + '_use_internal_dependency_generator\s+0')
+patch_fuzz_override_regexp = re.compile(DEFINE_RE + '_default_patch_fuzz\s+(\d+)')
 
 # See https://bugzilla.redhat.com/488146 for details
 indent_spaces_regex = re.compile('( \t|(^|\t)([^\t]{8})*[^\t]{4}[^\t]?([^\t][^\t.!?]|[^\t]?[.!?] )  )')
@@ -162,6 +171,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         buildroot_clean = {'clean': False, 'install' : False}
         depscript_override = False
         depgen_disabled = False
+        patch_fuzz_override = False
         indent_spaces = 0
         indent_tabs = 0
         files_has_defattr = False
@@ -264,7 +274,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if res:
                 pnum = res.group(1) or 0
                 for tmp in applied_patch_p_regex.findall(line) or [pnum]:
-                    pnum = int(tmp);
+                    pnum = int(tmp)
                     applied_patches.append(pnum)
                     if ifarch_depth > 0:
                         applied_patches_ifarch.append(pnum)
@@ -343,6 +353,13 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         printWarning(pkg, 'hardcoded-path-in-buildroot-tag',
                                      res.group(1))
 
+                res = buildarch_regex.search(line)
+                if res:
+                    if res.group(1) != "noarch":
+                        printError(pkg, 'buildarch-instead-of-exclusivearch-tag', res.group(1))
+                    else:
+                        package_noarch[current_package] = True
+
                 res = packager_regex.search(line)
                 if res:
                     printWarning(pkg, 'hardcoded-packager-tag', res.group(1))
@@ -352,10 +369,6 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                     if not res.group(1).startswith('%'):
                         printWarning(pkg, 'hardcoded-prefix-tag', res.group(1))
 
-                res = noarch_regex.search(line)
-                if res:
-                    package_noarch[current_package] = True
-
                 res = prereq_regex.search(line)
                 if res:
                     printError(pkg, 'prereq-use', res.group(2))
@@ -414,6 +427,9 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                 if not depgen_disabled:
                     depgen_disabled = \
                         depgen_disable_regex.search(line) is not None
+                if not patch_fuzz_override:
+                    patch_fuzz_override = \
+                        patch_fuzz_override_regexp.search(line) is not None
 
             if current_section == 'files':
 
@@ -489,6 +505,9 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         if depscript_override and not depgen_disabled:
             printWarning(pkg, 'depscript-without-disabling-depgen')
 
+        if patch_fuzz_override:
+            printWarning(pkg, 'patch-fuzz-is-changed')
+
         if indent_spaces and indent_tabs:
             pkg.current_linenum = max(indent_spaces, indent_tabs)
             printWarning(pkg, 'mixed-use-of-spaces-and-tabs',
@@ -561,7 +580,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         if clen is not None or cmd5 is not None:
                             # Not using path from urlparse results to match how
                             # rpm itself parses the basename.
-                            pkgfile = pkg.files()[url.split("/")[-1]]
+                            pkgfile = pkg.files().get(url.split("/")[-1])
                             if pkgfile:
                                 if clen is not None and pkgfile.size != clen:
                                     printWarning(pkg, 'file-size-mismatch',
@@ -627,6 +646,11 @@ by something like %{_tmppath}/%name-root.''',
 '''The Packager tag is hardcoded in your spec file. It should be removed, so
 as to use rebuilder's own defaults.''',
 
+'buildarch-instead-of-exclusivearch-tag',
+'''Use ExclusiveArch instead of BuildArch (or BuildArchitectures)
+to restrict build on some specific architectures.
+Only use BuildArch with noarch''',
+
 'hardcoded-prefix-tag',
 '''The Prefix tag is hardcoded in your spec file. It should be removed, so as
 to allow package relocation.''',
@@ -793,6 +817,12 @@ contents.''',
 '''The MD5 hash of the file in the package does not match the MD5 hash
 indicated by peeking at its URL.  Verify that the file in the package has the
 intended contents.''',
+
+'patch-fuzz-is-changed',
+'''The internal patch fuzz value was changed, and could hide patchs issues, or
+could lead to applying a patch at the wrong location. Usually, this is often the 
+sign that someone didn't check if a patch is still needed and do not want to rediff
+it. It is usually better to rediff the patch and try to send it upstream.'''
 )
 
 # SpecCheck.py ends here
