@@ -4,7 +4,6 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 00:03:24 1999
-# Version       : $Id: TagsCheck.py 1835 2011-02-05 08:21:19Z scop $
 # Purpose       : Check a package to see if some rpm tags are present
 #############################################################################
 
@@ -593,8 +592,8 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         is_source = pkg.isSource()
         for d in deps:
             value = apply(Pkg.formatRequire, d)
-            if use_epoch and d[1] and d[2][0] is None \
-                    and d[0][0:7] != 'rpmlib(':
+            if use_epoch and d[1] and d[2][0] is None and \
+                    not d[0].startswith('rpmlib('):
                 printWarning(pkg, 'no-epoch-in-dependency', value)
             for r in INVALID_REQUIRES:
                 if r.search(d[0]):
@@ -603,16 +602,18 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             if d[0].startswith('/usr/local/'):
                 printError(pkg, 'invalid-dependency', d[0])
 
-            if not devel_depend and not is_devel and not is_source and \
-                    FilesCheck.devel_regex.search(d[0]):
-                printError(pkg, 'devel-dependency', d[0])
-                devel_depend = True
-            if is_source and lib_devel_number_regex.search(d[0]):
-                printError(pkg, 'invalid-build-requires', d[0])
-            if not is_source and not is_devel:
-                res = lib_package_regex.search(d[0])
-                if res and not res.group(1) and not d[1]:
-                    printError(pkg, 'explicit-lib-dependency', d[0])
+            if is_source:
+                if lib_devel_number_regex.search(d[0]):
+                    printError(pkg, 'invalid-build-requires', d[0])
+            elif not is_devel:
+                if not devel_depend and FilesCheck.devel_regex.search(d[0]):
+                    printError(pkg, 'devel-dependency', d[0])
+                    devel_depend = True
+                if not d[1]:
+                    res = lib_package_regex.search(d[0])
+                    if res and not res.group(1):
+                        printError(pkg, 'explicit-lib-dependency', d[0])
+
             if d[1] == rpm.RPMSENSE_EQUAL and d[2][2] is not None:
                 printWarning(pkg, 'requires-on-release', value)
             self._unexpanded_macros(pkg, 'dependency %s' % (value,), value)
@@ -735,17 +736,19 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                             expected.append(
                                 extension_regex.sub('', expected[0]))
                         if ret.group(1) not in expected:
-                            if len(expected) == 1: expected = expected[0]
+                            if len(expected) == 1:
+                                expected = expected[0]
                             printWarning(pkg, 'incoherent-version-in-changelog',
                                          ret.group(1), expected)
 
-            if clt: changelog = changelog + clt
+            if clt:
+                changelog = changelog + clt
             if use_utf8 and not Pkg.is_utf8_str(' '.join(changelog)):
                 printError(pkg, 'tag-not-utf8', '%changelog')
 
             clt = pkg[rpm.RPMTAG_CHANGELOGTIME][0]
             if clt:
-                clt -= 12 * 3600 # rpm timestamps appear to be at noon
+                clt -= clt % (24*3600) # roll back to 00:00:00, see #246
                 if clt < oldest_changelog_timestamp:
                     printWarning(pkg, 'changelog-time-overflow',
                                  time.strftime("%Y-%m-%d", time.gmtime(clt)))
